#include <iostream>
#include <limits>

#include "format.hpp"
#include "abbn.hpp"

namespace abbn {
 static const int structure0[] = {
  1, 6, 10, 16, 22, 28, 36, 44, 53, 62, 71, 83, 95, 107, 119, 131, 143, 155,
  167, 179, 191, 203, 215, 227, 239, 251, 263, 275, 300 
 };

 const int* const structure = structure0 + 1;
 const int  ngroups = 28;
 const int mingroup = -1;
 const int maxgroup = 26;

 int group(int wide_group, int narrow_group) {
  return structure[wide_group] - 1 + narrow_group;
 };
};


static fortran_format::ast header_ast1("4X,A9,4X,A4,1X,2(4X,I4),5X,E12.5,6X,E13.5");
static fortran_format::ast header_ast2("4X,I3,10X,I4,2(5X,I3),6X,A30");

iabbn& iabbn::next_line() {
 ignore(std::numeric_limits<std::streamsize>::max(), '\n');
 skip_comments();
 return *this;
};

iabbn& iabbn::skip_comments() {
 if (good() && peek() == '*') next_line();
 return *this;
};

iabbn::iabbn(): reader1(header_ast1, *rdbuf()), reader2(header_ast2, *rdbuf()) {
 reader.s = rdbuf();
};

iabbn::iabbn(const char* filename, std::ios_base::openmode mode)
 : std::ifstream(filename, mode), 
   reader1(header_ast1, *rdbuf()), 
   reader2(header_ast2, *rdbuf()) {
 reader.s = rdbuf();
};

iabbn& iabbn::read_header() {
 skip_comments();
 reader1.begin() >> h.nam >> h.bib >> h.mf >> h.mt >> h.awr >> h.tem;
 reader1.end();
 reader2.begin();
 reader2 >> h.lv >> h.lt >> h.lc >> h.ls;
 char lf[31];
 reader2 >> lf;
 reader2.end();

 if (rdbuf()->sbumpc() == EOF) {
  setstate(eofbit);
  return *this;
 };
 rdbuf()->sungetc();

 // Extract format string
 char* f = lf;
 f = lf + 30;
 while (f > lf && *f != ')') --f;
 if (f > lf) *f = 0;
 f = lf;
 while (*f && *f != '(') ++f;
 if (*f) ++f;
 else f = lf;
 
 // Ensure uppercase
 for (char* c = f; *c; ++c)
  if (*c >= 'a' && *c <= 'z')
   *c += 'A' - 'a';

 h.lf.parse(f);
 reader.t = h.lf;
 reader.reset();
 next_line();
 return *this;
};

int iabbn::read_group() {
 if (h.lv == 20) {
  int wg, ng;
  *this >> wg >> ng;
  return abbn::group(wg, ng);
 };

 int g;
 *this >> g;
 return g;
};

bool iabbn::find(boost::function<bool (abbn::header)> test) {
 std::streampos start = tellg();
 bool overturn = false;
 while (!overturn || tellg() < start) {
  read_header();
  if (eof()) {
   overturn = true;
   clear();
   seekg(0, beg);
   continue;
  };
  if (!*this) return false;
  if (test(h)) return true;
  for (int i = 0; i < h.lt; ++i)
   next_line();
 };
 return false;
};
