package bnab;

use Exporter;
use Fortran::Format;

sub Fortran::Format::Edit::X::read_once {
 my $self = shift;
 $self->writer->position(relative => 1);
 ""
};

sub Fortran::Format::Edit::E::read_once {
 my $self = shift;
 return undef unless $self->writer->want_more;
 $self->writer->put($self->writer->read($self->{width}));
 1;
};

@ISA = qw(Exporter);
@EXPORT_OK = qw(ssfactors);

%reaction_numbers = (
 total0  =>  -1,
 total   =>   1,
 elastic =>   2,
 inelastic => 4,
 fission =>  18,
 capture => 102,
);

%group_structure = (
 -1 =>  5,
  0 =>  4,
  1 =>  6,
  2 =>  6,
  3 =>  6,
  4 =>  8,
  5 =>  8,
  6 =>  9,
  7 =>  9,
  8 =>  9,
  9 => 12,
 10 => 12,
 11 => 12,
 12 => 12,
 13 => 12,
 14 => 12,
 15 => 12,
 16 => 12,
 17 => 12,
 18 => 12,
 19 => 12,
 20 => 12,
 21 => 12,
 22 => 12,
 23 => 12,
 24 => 12,
 25 => 12,
 26 => 25
);

$header1 = Fortran::Format->new('4X,A9,4X,A6,3X,I3,5X,I3,6X,E12.5,6X,E12.5');
$header2 = Fortran::Format->new('4X,I3,10X,I4,5X,I3,5X,I3,6X,A30');

sub make_hash {
 die 'make_hash: array lengths are unequal: ', 
     scalar @{$_[0]}, ' != ', scalar @{$_[1]}
  if @{$_[0]} != @{$_[1]};
 my %result;
 for (my $i = 0; $i < @{$_[0]}; ++$i) {
  $result{$_[0][$i]} = $_[1][$i];
 };
 return %result;
};

sub comment {
 return $_[0] =~ /^\*/;
};

sub read_line {
 my $file = shift;
 local $_;
 while (<$file>) {
  return $_ unless comment $_;
 };
 return undef;
};

sub read_header {
 my $file = shift;
 my ($l1, $l2) = (read_line($file), read_line($file));
 return undef unless defined($l1) && defined($l2);
 my %result = (
  make_hash(
   [ 'nam', 'bib', 'mf', 'mt', 'awr', 'tem' ], 
   [ $header1->read($l1, (1) x 6) ]
  ),
  make_hash(
   [ 'lv', 'lt', 'lc', 'ls', 'lf' ],
   [ $header2->read($l2, (1) x 5) ]
  )
 );
 $result{'lf'} =~ s/[()]//g;
 $result{'lf'} =~ s/^ +//;
 $result{'lf'} =~ s/\r//;
 $result{'lf'} =~ s/ *$//;
 return %result;
};

sub read_body {
 my $file = shift;
 my $format = Fortran::Format->new(shift());
 my $lc = shift;
 my $lt = shift;
 my @result;
 my $started = 0;
 local $_;
 while (<$file>) {
  last unless comment $_;
 };
 do {
  s/\r//;
  push @result, [ $format->read($_, (1) x $lc) ];
  $_ = <$file>;
 } while (--$lt > 0);
 return @result;
};

#sub dilutions {
# my @result = (0);
# for ($i = -3; $i <= 21; ++$i) {
#  push @result, 10**($i/3);
# };
# return @result;
#};

sub dilutions {
 return (
  0.001, 0.1, 0.215, 0.465, 1, 2.15, 4.65, 10, 21.5, 46.5, 100, 215, 465, 1.00E+03,
  2.15E+03, 4.65E+03, 1.00E+04, 2.15E+04, 4.65E+04, 1.00E+05, 2.15E+05,
  4.65E+05, 1.00E+06, 2.15E+06, 4.65E+06, 1.00E+07
 );
};

sub test_header {
 my ($header, $key) = @_;
 local $_;
 foreach (keys %{$key}) {
  if (/nam|bib|lf/) {
   return undef if $$key{$_} ne $$header{$_};
  } elsif (/awr|tem/) {
   return undef if (abs($$header{$_}/$$key{$_} - 1) > 1E-6);
  } else {
   return undef if $$key{$_} != $$header{$_};
  };
 };
 return 1;
};

sub seek_table {
 my ($file, %key) = @_;
 my $startpos = tell $file;
 my $looped = 0;
 my $pos;
 local $_;
 while (!($looped && $pos >= $startpos)) {
  $pos = tell $file;
  my %header = read_header $file;
  if (test_header \%header, \%key) {
   $$_[0] = $pos if defined $_[0];
   return %header;
  };
  while (<$file>) {
   last unless comment $_;
  };
  seek $file, ($header{'lt'} - 1) * length($_), 1;
  if (eof $file) {
   $looped = 1;
   seek $file, 0, 0;
   $pos = tell $file;
  };
 };
 return ();
};

sub ssfactors {
 my $file = shift;
 my $mt = $reaction_numbers{shift()};
 my $tem = defined($_[0]) ? shift : 300;

 my $f;
 if (ref \$file eq 'GLOB') { $f = $file }
 else { 
  open $f, '<', $file or die "bnab::ssfactors: cannot open file `$file': $!\n";
 };

 my %header = seek_table $f, mf => 304, mt => $mt, tem => $tem;
 my @result;

 if (%header) {
  @result = read_body $f, @header{'lf','lc','lt'};
  my $i = $header{'lv'} == 20 ? 1 : 0;
  @result = (
   ([ (1) x (@{$result[0]}-$i - 1) ]) x ($result[0][$i]-1),
   (map { [ @$_[$i+1 .. $#{$_}] ] } @result)
  );
 } else {
  die "Cannot find section MF = 304, MT = $mt, TEM = $tem in "
      . (ref \$file eq 'GLOB' ? 'ABBN file' : $file)
      . "\n";
 };

 close $f unless ref \$file eq 'GLOB';

 return @result;
};

sub xsections {
 my $file = shift;

 my $f;
 if (ref \$file eq 'GLOB') { $f = $file }
 else {
  open $f, '<', $file or die "bnab::xsections: cannot open file `$file': $!\n";
 };

 my %header = seek_table $f, mf => 301, mt => 0;
 my @result;
 if (%header) {
  my $start = $header{'lv'} == 20 ? 2 : 1;
  @result = map {
   {
    total     => $$_[$start],
    capture   => $$_[$start + 1],
    fission   => $$_[$start + 2],
    elastic   => $$_[$start + 3],
    inelastic => $$_[$start + 4],
   }
  } read_body $f, @header{'lf','lc','lt'};
 } else {
  die 'Cannot find section MF = 301, MT = 0 in ' 
      . (ref \%file eq 'GLOB' ? 'ABBN file' : $file)
      . "\n";
 };

 close $f unless ref \$file eq 'GLOB';

 return @result;
};

1;
