#ifndef FORMATTED_EXCEPTION_HEADER
#define FORMATTED_EXCEPTION_HEADER

#include <stdexcept>
#include <boost/format.hpp>

#include <iostream>

#include <string.h>

class formatted_exception: public std::exception {
 private:
  mutable char* message;
  mutable boost::format* format;
 public:
  formatted_exception(const char* message)
   : message(NULL), format(new boost::format(message)) {};

  formatted_exception(const std::string& message)
   : message(NULL), format(new boost::format(message)) {};

  formatted_exception(const formatted_exception& fe) {
   if (fe.message) {
    message = strdup(fe.message);
    format = NULL;
   } else {
    message = NULL;
    format = new boost::format(*fe.format);
   };
  };

  virtual ~formatted_exception() throw() {
   free(message);
   delete format;
  };

  template <typename T> formatted_exception& operator<<(const T& t) {
   (*format) % t;
   return *this;
  };

  const char* what() const throw() {
   if (message) return message;
   message = strdup(format->str().c_str());
   delete format;
   format = NULL;
   return message;
  };
};

#endif
