#include <gsl/gsl_errno.h>
#include <gsl/gsl_linalg.h>

#include "gsl.hpp"

std::vector<std::vector<double> > system_solve(
  const double* matrix, 
  const double* rhs, 
  size_t size,
  size_t n
) { 
 gsl_matrix_view m = gsl_matrix_view_array(
   const_cast<double*>(matrix), size, size
 );
 gsl_vector_view b = gsl_vector_view_array(
   const_cast<double*>(rhs), size
 );

 gsl_vector* x = gsl_vector_calloc(size);
 gsl_permutation* p = gsl_permutation_alloc(size);
 int s;

 std::vector<std::vector<double> > result;

 int errcode = gsl_linalg_LU_decomp(&m.matrix, p, &s);
 size_t i = 0;
 if (errcode == 0) {
//  for (size_t j = 0; j < size; ++j)
//   if (gsl_matrix_get(&m.matrix, j, j) == 0)
//    throw system_degenerated_error("system_solve: system is degenerated");

  result.resize(n, std::vector<double>(size));
  for (; i < n; ++i) {
   errcode = gsl_linalg_LU_solve(
     &m.matrix, 
     p, 
     &gsl_vector_view_array(const_cast<double*>(rhs+i*size), size).vector, 
     x
   );
   if (errcode) break;
   for (size_t j = 0; j < size; ++j)
    result[i][j] = gsl_vector_get(x, j);
  };
 };

 gsl_permutation_free(p);
 gsl_vector_free(x);

 if (errcode)
  throw std::runtime_error(gsl_strerror(errcode));

 return result;
};

std::vector<double> system_solve(
  const double* matrix, const double* rhs, size_t size
) {
 return system_solve(matrix, rhs, size, 1)[0];
};
