#ifndef GSL_HEADER
#define GSL_HEADER

#include <stdexcept>
#include <vector>

class system_degenerated_error: public std::runtime_error {
 public:
  system_degenerated_error(const std::string& message)
  : std::runtime_error(message) {};
};

// Solves the system of equations matrix*x = rhs
// Parameters:
//  matrix -- one-dimensional system matrix (matrix[i*n+j] is the same as
//            matrix[i][j] in two-dimensional notation)
//  rhs -- right-hand side of the matrix equation
//  size -- sizeof(rhs)/sizeof(double) and sqrt(sizeof(matrix)/sizeof(double))
std::vector<double> system_solve(
  const double* matrix, const double* rhs, size_t size
);
// The same, but for n rhses
std::vector<std::vector<double> > system_solve(
  const double* matrix, const double* rhs, size_t size, size_t n
);

#endif
