#ifndef POLYNOME_HEADER
#define POLYNOME_HEADER

#include <vector>

#ifdef DEBUG
#include <iostream>
#endif

class polynome: public std::vector<double> {
 public:
  polynome();
  polynome(double);
  polynome(size_t, double = 0);
  polynome(const std::vector<double>&);
  polynome(const polynome&);

  polynome operator-() const;
  polynome operator+(double) const;
  polynome operator-(double) const;
  polynome operator*(double) const;
  polynome operator/(double) const;

  polynome operator+(const polynome&) const;
  polynome operator-(const polynome&) const;
  polynome operator*(const polynome&) const;
  polynome operator/(const polynome&) const;
  polynome operator%(const polynome&) const;

  polynome& operator+=(double);
  polynome& operator-=(double);
  polynome& operator*=(double);
  polynome& operator/=(double);

  polynome& operator+=(const polynome&);
  polynome& operator-=(const polynome&);
  polynome& operator*=(const polynome&);
  polynome& operator/=(const polynome&);
  polynome& operator%=(const polynome&);

  bool operator==(double) const;
  bool operator!=(double) const;
  bool operator==(const polynome&) const;
  bool operator!=(const polynome&) const;

  double operator()(double) const;

  inline size_t power() const { return size()-1; };
  std::vector<double> roots() const;
  polynome derivate() const;
  polynome& nderivate();
  polynome shift(double dx) const;
  polynome& nshift(double dx);

  void normalize();
};

polynome operator+(double, const polynome&);
polynome operator-(double, const polynome&);
polynome operator*(double, const polynome&);

bool operator==(double, const polynome&);
bool operator!=(double, const polynome&);

#ifdef DEBUG
std::ostream& operator<<(std::ostream&, const polynome&);
#endif

#endif
