#include "program_options.hpp"

#include <string>
#include <vector>

#include <iostream>

using namespace boost::program_options;

std::vector<std::string> split(const char* s, char d) {
 std::vector<std::string> result;
 if (!s)
  return result;
 const char* c = s;
 do {
  s = c;
  while (*c != 0 && *c != d) ++c;
  result.push_back(std::string(s, (c-s)/sizeof(char)));
 } while (*c++ != 0);
 return result;
};

void parse_environment(
  std::map<std::string, variable_value>& v,
  const char* name,
  const char* envname,
  value_semantic* value
) {
 std::vector<std::string> envs = split(envname, ',');
 for (size_t i = 0; i < envs.size(); ++i) {
  char* env = getenv(envs[i].c_str());
  if (env) {
   // TODO: support arrays in environment variables 
   // (got to find a good separator or provide an interface to change it)
   boost::any x;
   value->parse(x, std::vector<std::string>(1, env), false);
   v[name].value() = x; // thus parse() does not compain about defaulted() 
			// values
   break;
  };
 };
};

inline void add(options_description_easy_init& o, const my_option_description& x) {
 std::string name = x.longname;
 if (x.shortname != 0)
  name.append(1, ',').append(1, x.shortname);
 o(name.c_str(), x.value ? x.value : new untyped_value(true), x.description);
};

inline void add(
  options_description_easy_init o, const my_positional_option_description& x
) {
 o(x.name, x.value ? x.value : new untyped_value(true), "");
};

variables_map parse_args(
  int argc, char** argv,
  const my_option_description* desc,
  size_t descsize,
  const my_positional_option_description* posdesc,
  void (*usage)(const options_description&)
) {
 descsize /= sizeof(my_option_description);

 variables_map result;

 // Parse command line
 options_description general;
 options_description_easy_init general_odei = general.add_options();
 options_description hidden;
 for (size_t i = 0; i < descsize; ++i)
  add(general_odei, desc[i]);
 if (!posdesc)
  store(command_line_parser(argc, argv).options(general).run(), result);
 else {
  add(hidden.add_options(), *posdesc);
  options_description all(general);
  all.add(hidden);
  positional_options_description positional;
  positional.add(posdesc->name, posdesc->number);
  store(
    command_line_parser(argc, argv).options(all).positional(positional).run(),
    result
  );
 };

 // Parse environment
 for (size_t i = 0; i < descsize; ++i)
  if (desc[i].envname && 
     (!result.count(desc[i].longname) || result[desc[i].longname].defaulted()))
   parse_environment(result, desc[i].longname, desc[i].envname, desc[i].value);
 if (posdesc && posdesc->envname && 
    (!result.count(posdesc->name) || result[posdesc->name].defaulted()))
  parse_environment(result, posdesc->name, posdesc->envname, posdesc->value);

 if (usage && result.count("help"))
  usage(general);

 return result;
};
