#ifndef SUBGROUPS_HEADER
#define SUBGROUPS_HEADER

#include <vector>
#include <stdexcept>

#include <boost/function.hpp>

#ifdef DEBUG
#include <iomanip>
#include "debug.hpp"
#endif

struct subgroup {
 double probability, total;
 std::vector<double> partial;
};

namespace subgroups {
 struct knot {
  double dilution, total1, total2; // FIXME: total1 is superfluous
  std::vector<double> partial;
 };

 class bad_accuracy: public std::exception {
  public:
   double error;
   std::vector<subgroup> result;
   char* message;

   bad_accuracy(double error, const std::vector<subgroup>& result);
   ~bad_accuracy() throw() {
    if (message) delete[] message;
   };

   const char* what() const throw() { return message; };
 };

 double mean(const std::vector<subgroup>& subgroups);
 double mean(const std::vector<subgroup>& subgroups, size_t x);
 double blocked_xsection(
   const std::vector<subgroup>& subgroups, double dilution
 );
 double blocked_xsection(
   const std::vector<subgroup>& subgroups, double dilution, size_t x
 );

 double relative_error(double x, double y);
};

template <typename T1, typename T2, typename T3>
struct triple {
 T1 first;
 T2 second;
 T3 third;
};

#ifdef DEBUG
 static std::ostream& operator<<(
   std::ostream& s, const std::vector<subgroup>& subgroups
 ) {
  std::streamsize precision = s.precision(5);
  std::ios::fmtflags flags = s.setf(std::ios::scientific | std::ios::right);
  const int  field_width = 12;
  for (size_t i = 0; i < subgroups.size(); ++i) {
   s << (i+1)
    << ' ' << std::setw(field_width) << subgroups[i].probability
    << ' ' << std::setw(field_width) << subgroups[i].total;
   for (size_t x = 0; x < subgroups[i].partial.size(); ++x)
    s << ' ' << std::setw(field_width) << subgroups[i].partial[x];
   s << '\n';
  };
  s.setf(flags);
  s.precision(precision);
  return s;
 };
#endif

std::vector<subgroup> compute_subgroups(
  double total0,
  const std::vector<double>& partials,
  const std::vector<subgroups::knot>& knots,
  double maxerror,
  double* error = NULL,
  int ignore = -1
);

std::vector<subgroup> compute_subgroups_K(
  double total0,
  const std::vector<double>& partials,
  const std::vector<subgroups::knot>& knots,
  size_t K,
  double maxerror,
  double *error = NULL,
  int ignore = -1
);

#endif
