package subgroups;

use YAML::XS;
use File::Slurp qw(slurp);

sub make_hash {
 die 'make_hash: array lengths are unequal: ', 
     scalar @{$_[0]}, ' != ', scalar @{$_[1]}
  if @{$_[0]} != @{$_[1]};
 my %result;
 for (my $i = 0; $i < @{$_[0]}; ++$i) {
  $result{$_[0][$i]} = $_[1][$i];
 };
 return %result;
};

sub read {
 my $file = shift;
 my @captions = ('probability', 'total', @_);
 my $group;
 my @result;
 my $f;
 if (ref $file eq 'GLOB') { $f = $file }
 else { open $f, '<', $file };
 while (<$f>) {
  split;
  $group = shift if /^ {0,2}\d/;
  my $subgroup = shift;
  pop @_ if $subgroup == 1; # remove error
  push @_, 0 if @_ < @captions; # dummy inelastic cross-section
  $result[$group-1][$subgroup-1] = { make_hash \@captions, \@_ };
 };
 close $f unless ref $file eq 'GLOB';
 return @result;
};

sub yaml_decode {
 return
  map {
   my $partials = $$_{'partials'};
   [
    map {
     my %subgroup = ('probability' => $$_[0], 'total' => $$_[1]);
     for (my $i = 0; $i < @{$partials}; ++$i) {
      $subgroup{$$partials[$i]} = $$_[$i+2];
     };
     \%subgroup;
    } @{$$_{'subgroups'}}
   ]
  } @{$_[0]}
};

sub read_yaml {
 return yaml_decode YAML::XS::Load ref $_[0] eq 'GLOB' 
                                   ? join '', <$_[0]> 
                                   : scalar slurp $_[0];
};

1;
